# -*- coding: utf-8 -*-
# Author:   $Author: merkosh $
# Revision: $Rev: 180 $
############################################################################
#    Copyright (C) 2005 by Uwe Mayer                                       #
#    merkosh@hadiko.de                                                     #
#                                                                          #
#    This program is free software; you can redistribute it and/or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################

#-- imports --------------------------------------------------------------------
#-- Qt imports
from qt import SIGNAL, QListView, QSize, QTimer
from qt import PYSIGNAL, QApplication


#-- Python imports
import logging

from types import ListType


#-- own imports
from Settings import getPreferences
from AbstractFile import AbstractFileRecord
from misc import Date2YMD
from lmcqt import ListViewSort


#-- preferences ----------------------------------------------------------------
pref = getPreferences(__name__)



class ListViewRecord(QListView):
    def __init__(self, parent=None, name=None, fl=0):
        """displays records in a list view and allows incremental search"""
        QListView.__init__(self, parent, name, fl)

        # create logging
        self.log = logging.getLogger( self.__class__.__name__ )
        self.log.setLevel(pref['logLevel'])

        # define layout
        self.setMinimumSize(QSize(180,0))
        self.addColumn("No")
        self.addColumn("Title")

        # rudimentry properties
        self.setAllColumnsShowFocus( True )
        self.setSorting( pref['sortColumn'] )
        self.setShowSortIndicator( True )
        self.setColumnWidthMode( 0, QListView.Maximum )
        self.setColumnWidthMode( 1, QListView.Maximum )
        self.setResizeMode( QListView.LastColumn )
        self.setSelectionMode( QListView.Single )
        self.setDefaultRenameAction( QListView.Accept )

        # runtime variables
        self.obj = None           # keeps track of the last item added
        self.items = {}                 # maps ListViewItem -> record

        # normal search
        self.wrappedSearch = False
        self.firstSearch = True

        # incremental search string
        self.incSearch = ""
        self.incSearchHistory = []
        self.incWrappedSearch = False
        self.searchOffTimer = QTimer()

        # grouping
        self.groupMap = dict()          # maps string(value) -> parent object
        self.currentGroup = None

        # connect signals
        self.connect( self.searchOffTimer, SIGNAL("timeout()"), self.onSearchTimeout )
        self.connect( self, SIGNAL("itemRenamed(QListViewItem*,int,const QString&)"), self.onItemRenamed )


    #-- python interface -------------------------------------------------------
    def __delitem__(self, y):
        """removes ListViewItem(s) by reference"""
        self.log.debug("deleting record %s", y)
        if (not type(y) is ListType): y = [y]
        for obj in y:
            self.takeItem(obj)
            del self.items[obj]


    def values(self):
        """returns an unsorted list of all records"""
        return self.items.values()


    def __len__(self):
        """returns number of elements in the list view"""
        return len(self.items.keys())
    

    #-- grouping functions -----------------------------------------------------
    def onGroup(self, group):
        """called when an online re-grouping is necessary"""
        self.log.info("Regrouping records according to %s", group)

        # remember current item (changes when taking out items)
        tmpCurrentItem = self.currentItem()

        # previous parents need to be removed
        oldParents = self.groupMap.values()
        for p in oldParents:
            self.takeItem(p)           # (!!) self must be parent to p
        # remove list of parents
        self.groupMap.clear()

        # regroup
        self.currentGroup = group
        for obj in self.items.keys():
            oldParent = obj.parent() or self
            oldParent.takeItem(obj)
            assert(self.items.has_key(obj))
            newParent = self.parentGroup(self.items[obj])
            newParent.insertItem(obj)
    
        # decorate listView on grouping
        self.setRootIsDecorated(group != None)
        # shrink column 0 to minimum width
        self.setColumnWidth(0, 0)
        # restore previous current item
        if (self.items.has_key(tmpCurrentItem)):
            self.setCurrentItem(tmpCurrentItem)
        #  remove old parents
        del oldParents
            
        
    def parentGroup(self, rec):
        """returns the parent for a QListViewItem"""
        if (self.currentGroup != None):
            # key values to self.groupMap *must* be strings 
            value = unicode(rec[self.currentGroup])
            # group by date
            if (self.currentGroup == "date"):
                # this *is* an int, because of the Qt object its contained in
                ymd = Date2YMD(int(value))
                value = "%02d-%02d-%04d"%(ymd[2], ymd[1], ymd[0])
                
            # all other fields: if no parent exists yet
            if (not self.groupMap.has_key(value)):
                self.log.debug("Creating parent group %s", value)
                parent = ListViewSort(self, value, "(1)")
                parent.setSelectable(False)
                self.groupMap[value] = parent

            # update child count
            else:
                parent = self.groupMap[value]
                oldVal = unicode(parent.text(1)) 
                parent.setText(1, "(%d)"%(int(oldVal[1:-1])+1,))

            # return new (or old) parent
            return parent
        
        # default: no grouping
        return self         


    #-- own interface ----------------------------------------------------------
    def add(self, rec):
        """W.add(rec) -- append record(s)"""
        # add all new records
        if (not type(rec) is ListType): rec = [rec]
        obj = None
        for r in rec:
            if (not isinstance(r, AbstractFileRecord)):
                raise(TypeError("Record not instance of AbstractFileRecord"))
            parent = self.parentGroup(r)
            obj = ListViewSort(parent, *self.compileTitles(r) )
            obj.setRenameEnabled(0, True)
            self.items[obj] = r
        # keep track of last item added
        self.obj = obj


    def getCurrentRecord(self):
        """returns the current record"""
        # this test fails:
        # when regrouping the self.currentItem() may be a QListViewItem that
        # has been already taken and removed, thus causing a segfault
        # better to test for inclusion in self.items
        #         if (self.currentItem() in self.groupMap.values()):
        #             return None
        if (not self.items.has_key(self.currentItem())):
            return None
        return self.items[self.currentItem()]


    def setCurrentRecord(self, record):
        """sets values of the current record"""
        currentObj = self.currentItem()
        self.items[currentObj] = record        
        self.onModified()


    def findNext(self, text, field, options):
        """continues search for a text in a specific field"""
        # wether to clear search status
        if (text is None):
            self.log.debug("Clearing search status")
            self.firstSearch = True
            return

        self.log.info("Searching for %s in %s", text, field or "<all fields>")

        # runtime variables
        allFieldKeys = ('mediaLabel', 'source', 'borrower', 'originalTitle',
                        'translatedTitle', 'director', 'producer', 'country',
                        'category', 'actors', 'description', 'comments',
                        'languages', 'subtitles', 'year')

        noRecord = dict()
        for f in allFieldKeys: noRecord.setdefault(f, "")
        
        keyFound = None
        found = False
        visited = dict()
        obj = self.currentItem()        # start at current item


        def stepDirection(obj, directionForward):
            """given a listviewitem object it takes a step and returns the next element"""
            # searching forward
            if (directionForward):
                # step one down
                obj = obj.itemBelow()
                # stepped on parent group element: goto first child
                if (obj != None) and (not self.items.has_key(obj)):
                    obj = obj.firstChild()
            # searching backwards
            else:
                # step one upwards
                obj = obj.itemAbove()
                # stepped on parent group element
                if ((obj != None) and (not self.items.has_key(obj))):
                    # get previous element (may be group or node)
                    obj = obj.itemAbove()
                    # if is group element
                    if ((obj != None) and (not self.items.has_key(obj))):
                        # goto last child
                        obj = obj.firstChild()
                        assert(obj != None) # every group element has children
                        while (obj.nextSibling() != None): obj = obj.nextSibling()
            return obj
        

        # on first time use current item as starting point
        if ((obj != None) and (not self.firstSearch)):
            obj = stepDirection(obj, options['searchForward'])
        self.firstSearch = False

        # considder case-sensitivity
        if (not options['caseSensitive']):
            text = text.lower()

        while (not found):
            # on wrap over
            if (self.wrappedSearch):
                self.log.info("Continuing wrapped search")
                self.wrappedSearch = False
                if (options['searchForward']):
                    obj = self.firstChild()
                else:
                    obj = self.lastItem()
                    assert(self.items.has_key(obj))

            # at end of page (top or bottom)
            if (obj == None):
                QApplication.beep()

                # if all items have been searched
                if (len(visited.keys()) == len(self.items.keys())): break

                if (options['searchForward']):
                    self.log.info("Continue search at the top?")
                    self.emit(PYSIGNAL("searchTextChanged"), ("Continue search at the top?",))
                else:
                    self.log.info("Continue search at the bottom?")
                    self.emit(PYSIGNAL("searchTextChanged"), ("Continue search at the bottom?",))
                self.wrappedSearch = True
                return

            # search in all fields
            rec = (self.items.has_key(obj) and self.items[obj]) or noRecord
            if (field == None):
                for k in allFieldKeys:
                    keyFound = k
                    value = unicode(rec[k])
                    if (not options['caseSensitive']):
                        value = value.lower()
                    found = (value.find(text) != -1)
                    if (found): break
            # search in given field
            else:
                assert(rec.has_key(field))
                value = unicode(rec[field])
                if (not options['caseSensitive']):
                    value = value.lower()
                found = (value.find(text) != -1)

            # add object to visited items
            visited[obj] = 1

            # move on in direction
            if (not found):
                obj = stepDirection(obj, options['searchForward'])

        # focus on found object
        if (found):
            self.log.debug("Found match at %d in field %s", rec['number'], keyFound or field)
            self.emit(PYSIGNAL("searchTextChanged"), ("Found match at %d in field %s"%(rec['number'], keyFound or field),))
            self.focus(obj)
        # error if not found
        else:
            self.log.info("No match found")
            self.emit(PYSIGNAL("searchTextChanged"), ("No match found",))
            
            

    def changeNumber(self, mode, number):
        """called when the number of the current item needs be changed"""
        
        if (mode == "first"):
            # get list of all indexes
            indexList = []
            for r in self.items.values():
                indexList.append(int(r['number']))
            indexList.sort()

            # find first free index
            index = 1
            for i in indexList:
                if (i < index): continue
                elif (i == index): index += 1
                else: break

            # set index
            self.log.info("Replacing number with first free index %d", index)
            self.getCurrentRecord()['number'] = index
            
        elif (mode == "swap"):
            self.log.info("Swapping number with record %d", number)
            curr = self.getCurrentRecord()
            other = filter(lambda a: a['number'] == number, self.items.values())
            
            if (len(other) > 1):
                self.log.warning("Multiple record numbers %d (count=%d); swapping with all", number, len(other))
            if (len(other) == 0):
                self.log.warning("No record with number %d exist; not swapping", number)
                
            tmp = curr['number']
            curr['number'] = number
            for o in other: o['number'] = tmp
                
        elif (mode == "insert"):
            self.log.info("Inserting record at number %d", number)

            # get records that need re-indexing
            shifts = filter(lambda a: a['number'] >= number, self.items.values())
            # sort ascendingly; need only to increase indexes
            # until a missing index is found
            shifts.sort(lambda a,b: int(a['number']) -int(b['number']))
            index = number
            for r in shifts:
                # stop when there's a missing index
                if (r['number'] != index): break
                # increase number
                r['number'] += 1
                index += 1

            # set current item to the designated value
            self.getCurrentRecord()['number'] = number
                
        elif (mode == "set"):
            self.log.info("Setting record index to %d", number)
            self.getCurrentRecord()['number'] = number
            
        else:
            self.log.warning("Invalid mode in changing number: %s", mode)
        
        # update display
        for obj in self.items.keys():
            col = 0
            for text in self.compileTitles(self.items[obj]):
                obj.setText(col, text)
                col += 1
        self.sort()



    def focusFirst(self):
        """sets focus on the first record in the listview"""
        self.focus(self.firstChild())


    def focusNew(self):
        """sets focus on the last item added"""
        self.focus(self.obj)
    

    def focus(self, obj):
        """sets focus on one particular object"""
        if (obj != None):
            self.ensureItemVisible(obj)
            self.setCurrentItem(obj)    # sends currentChanged SIGNAL
            self.emit(SIGNAL("currentChanged(QListViewItem*)"), (obj,))
        self.setFocus()
          

    #-- helper functions -------------------------------------------------------
    def compileTitles(self, record):
        """return a text-tuple what forms the ListViewItem entries """
        num = unicode(record['number'])
        if (pref['zeroPrefix_patch']):
            num = u'0'*(4-len(num))+num
        return (num,
                record['mediaLabel'] or record['originalTitle'] or record['translatedTitle'])


    #-- slots ------------------------------------------------------------------   
    def onModified(self):
        """called when a record is modified in the form"""
        # update ListViewItem title
        index = 0
        record = self.items[self.currentItem()]
        for text in self.compileTitles(record):
            self.currentItem().setText(index, text)
            index += 1

        # resort into group if data is grouped
        if (self.currentGroup != None):
            # get current item and old and new parents
            item = self.currentItem()
            oldParent = item.parent() or self
            newParent = self.parentGroup(self.items[item])
            # if parent changed
            if (oldParent != newParent):
                # change parentship
                oldParent.takeItem(item)
                newParent.insertItem(item)
                # cosmetics
                oldParent.setOpen(False)
                # remove empty parents
                oldParentChildCount = int(unicode(oldParent.text(1)[1:-1]))
                if (oldParentChildCount == 1):
                    # remove item from screen
                    self.takeItem(oldParent)
                    # remove reference to parent from internal dict
                    del self.groupMap[unicode(oldParent.text(0))]
                    # remove parent object itself
                    del oldParent
                else:
                    # update child count
                    oldParent.setText(1, "(%d)"%(oldParentChildCount-1,))

                # focus on new child
                self.focus(item)
        


    def onItemRenamed(self, obj, col, text):
        """called when record number is modified in the list view"""
        try:
            number = int(unicode(text))
        except ValueError:
            pass
        else:
            self.items[obj]['number'] = number
        # update list view
        self.onModified()
        self.sort()


    #-- incremental search 
    def onSearchTimeout(self):
        """called when timeout on incremental search"""
        self.searchOffTimer.stop()      # stop running timer
        self.emit(PYSIGNAL("searchTextChanged"), ("Stopping incremental search",))
        self.log.debug("Clearing incremental search history")
        self.incSearch = ""
        self.incSearchHistory = []
        self.incWrappedSearch = False
      

    def keyPressEvent(self, e):
        """reacs on key-press-events for incremental search"""      
        # ignore keypresses when renaming
        # ignore keypresses when empty
        if (self.isRenaming() or (len(self) == 0)):
            e.ignore()
            return

        if (e.text() == ""):
            QListView.keyPressEvent(self, e)
            return
       
        # react on backspace
        if (e.key() == QListView.Key_Backspace):
            if (len(self.incSearchHistory) > 0):
                (self.incSearch, prevItem) = self.incSearchHistory.pop()
                self.emit(PYSIGNAL("searchTextChanged"), ("Text found: \"%s\""%self.incSearch,))
                self.setCurrentItem( prevItem )
                self.ensureItemVisible( prevItem )
            else:
                QApplication.beep()
                self.emit(PYSIGNAL("searchTextChanged"), ("Search text empty",))
            return

        # abort on escape
        elif (e.key() == QListView.Key_Escape):
            self.onSearchTimeout()
            return

        # reset timer
        self.searchOffTimer.start(5000, True)

        currItem = self.currentItem()
        currText = self.incSearch

        # next search on enter or return
        if (e.key() in [QListView.Key_Enter, QListView.Key_Return]):
            # test on rollover
            self.setCurrentItem(currItem.nextSibling())
            nextItem = self.findItem(self.incSearch, 1, QListView.Contains)

            # enter on empty search string behaves like DOWN key
            if (currText == ""):
                self.ensureItemVisible(self.currentItem())
                return
            
            # rollover
            if (not self.incWrappedSearch and
                ((self.itemPos(nextItem) < self.itemPos(currItem)) or (currItem == self.lastItem()))):
                self.setCurrentItem(currItem)
                self.incWrappedSearch = True
                self.emit(PYSIGNAL("searchTextChanged"), ("End of list: wrap over?",))
                QApplication.beep()
                return
            elif (self.incWrappedSearch):
                self.incWrappedSearch = False
                self.setCurrentItem(self.firstChild())

        else:
            self.incSearch += unicode(e.text())

        self.incSearchHistory.append( (currText,currItem) ) # remember current item for back-travel
        item = self.findItem(self.incSearch, 1, QListView.Contains)
            
        # text does not match
        if (item == None):
            self.emit(PYSIGNAL("searchTextChanged"), ("Text not found: \"%s\""%self.incSearch,))
            QApplication.beep()
            self.log.info("No match for \"%s\"", self.incSearch)
        # match found
        else:
            self.emit(PYSIGNAL("searchTextChanged"), ("Text found: \"%s\""%self.incSearch,))
            self.setCurrentItem(item)
            self.ensureItemVisible(item)
    


#-- testarea -------------------------------------------------------------------
if (__name__ == "__main__"):
    pass
